/*
Example application for capturing motion video version 0.0.3

Usage: mrecord <filename>

Options: none yet, edit the source code.
*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <linux/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>

#include <linux/videodev.h>
#include "buz.h"

static const int norm = VIDEO_MODE_NTSC;
static const int input = 1;
static const int width = 720;
static const int height = 480;
static const int count = 30 * 15;	/* 15 seconds should be convincing */

int
main(int nargs, char ** argv)
{
	int dev;
	int file;
	int res;
	int frame;
	int play_frame;
	int sync_frame;
	struct video_window vw;
	struct video_channel vc;
	struct video_mbuf vb;
	struct video_mmap vm;
	struct buz_codec_control bc;
	struct buz_sync bs;
	char * buff;

	/* check usage */
	if (nargs < 2) {
		fprintf(stderr, "usage: %s <filename>\n", argv[0]);
		exit(1);
	}

	/* open device */
	dev = open("/dev/bttv0", O_RDWR);
	if (dev < 0) {
		perror("/dev/bttv0");
		exit(1);
	}

	/* open file */
	file = open(argv[1], O_RDONLY);
	if (file < 0) {
		perror(argv[1]);
		exit(1);
	}

	/* select input */
	vc.channel = input;
	res = ioctl(dev, VIDIOCGCHAN, &vc);
	if (res < 0) {
		perror("VIDIOCGCHAN");
		exit(1);
	}
	vc.norm = norm;
	res = ioctl(dev, VIDIOCSCHAN, &vc);
	if (res < 0) {
		perror("VIDIOCSCHAN");
		exit(1);
	}

	/* set up window parameters */
	vw.x = 0;
	vw.y = 0;
	vw.width = width;
	vw.height = height;
	vw.chromakey = 0;
	vw.flags = 0;
	vw.clips = NULL;
	vw.clipcount = 0;
	res = ioctl(dev, VIDIOCSWIN, &vw);
	if (res < 0) {
		perror("VIDIOCSWIN");
		exit(1);
	}

	/* get frame buffer information and map it */
	res = ioctl(dev, VIDIOCGMBUF, &vb);
	if (res < 0) {
		perror("VIDIOCGMBUF");
		exit(1);
	}
	printf("%s: %d buffers.\n", argv[0], vb.frames);
	buff = mmap(0, vb.size, PROT_WRITE, MAP_SHARED, dev, 0);
	if (buff == MAP_FAILED) {
		perror("mmap");
		exit(1);
	}

	/* play the video */
	play_frame = 0;
	sync_frame = 0;
	vm.height = height;
	vm.width = width;
	vm.format = 1;
	while (sync_frame < count) {
		/* issue as many plays as we can */
		while ((play_frame - sync_frame) < vb.frames) {
			/* read it in */
			frame = play_frame++ % vb.frames;
			res = read(file, &bs, sizeof(bs));
			if (res == 0)
				break;
			if (res != sizeof(bs)
			    ||  read(file, buff + vb.offsets[frame], bs.length) != bs.length) {
				perror("read");
				exit(1);
			}
			printf("frame %d, length=%u seq=%u stamp=%ld.%06ld\n",
			       play_frame - 1, bs.length, bs.seq, bs.timestamp.tv_sec, bs.timestamp.tv_usec);

			vm.frame = frame;
#if 1
			res = ioctl(dev, BUZIOCMPLAY, &vm);
			if (res < 0) {
				perror("BUZIOCMPLAY");
				exit(1);
			}
#endif
		}

		/* sync on a frame */
		frame = sync_frame++ % vb.frames;
		bs.frame = frame;
#if 1
		res = ioctl(dev, BUZIOCSYNC, &bs);
		if (res < 0) {
			perror("BUZIOCSYNC");
			exit(1);
		}
#endif
	}

	res = close(file);
	if (res < 0) {
		perror(argv[1]);
		exit(1);
	}

	res = close(dev);
	if (res < 0) {
		perror("/dev/bttv0");
		exit(1);
	}

}
