/*
Example application for capturing motion video version 0.0.3

Usage: mrecord <filename>

Options: none yet, edit the source code.
*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <linux/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>

#include <linux/videodev.h>
#include "buz.h"

static const int norm = VIDEO_MODE_NTSC;
static const int input = 1;
static const int width = 720;
static const int height = 480;
static const int count = 30 * 15;	/* 15 seconds should be convincing */

int
main(int nargs, char ** argv)
{
	int dev;
	int file;
	int res;
	int frame;
	int grab_frame;
	int sync_frame;
	struct video_window vw;
	struct video_channel vc;
	struct video_mbuf vb;
	struct video_mmap vm;
	struct buz_codec_control bc;
	struct buz_sync bs;
	char * buff;

	/* check usage */
	if (nargs < 2) {
		fprintf(stderr, "usage: %s <filename>\n", argv[0]);
		exit(1);
	}

	/* open device */
	dev = open("/dev/bttv0", O_RDONLY);
	if (dev < 0) {
		perror("/dev/bttv0");
		exit(1);
	}

	/* open file */
	file = creat(argv[1], S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
	if (file < 0) {
		perror(argv[1]);
		exit(1);
	}

	/* select input */
	vc.channel = input;
	res = ioctl(dev, VIDIOCGCHAN, &vc);
	if (res < 0) {
		perror("VIDIOCGCHAN");
		exit(1);
	}
	vc.norm = norm;
	res = ioctl(dev, VIDIOCSCHAN, &vc);
	if (res < 0) {
		perror("VIDIOCSCHAN");
		exit(1);
	}

	/* set up window parameters */
	vw.x = 0;
	vw.y = 0;
	vw.width = width;
	vw.height = height;
	vw.chromakey = 0;
	vw.flags = 0;
	vw.clips = NULL;
	vw.clipcount = 0;
	res = ioctl(dev, VIDIOCSWIN, &vw);
	if (res < 0) {
		perror("VIDIOCSWIN");
		exit(1);
	}

	/* set up codec */
	bc.quality = 65535;
	strcpy(bc.comment, "Penguin Power");
	res = ioctl(dev, BUZIOCSCODEC, &bc);
	if (res < 0) {
		perror("BUZIOCSCODEC");
		exit(1);
	}

	/* get frame buffer information and map it */
	res = ioctl(dev, VIDIOCGMBUF, &vb);
	if (res < 0) {
		perror("VIDIOCGMBUF");
		exit(1);
	}
	printf("%s: %d buffers.\n", argv[0], vb.frames);
	buff = mmap(0, vb.size, PROT_READ, MAP_SHARED, dev, 0);
	if (buff == MAP_FAILED) {
		perror("mmap");
		exit(1);
	}

	/* capture the video */
	grab_frame = 0;
	sync_frame = 0;
	vm.height = height;
	vm.width = width;
	vm.format = 1;
	while (sync_frame < count) {
		/* issue as many grabs as we can */
		while ((grab_frame - sync_frame) < vb.frames
		       && grab_frame < count) {
			vm.frame = grab_frame % vb.frames;
			res = ioctl(dev, VIDIOCMCAPTURE, &vm);
			if (res < 0) {
				perror("VIDIOCMCAPTURE");
				exit(1);
			}
			++grab_frame;
			vm.format = 0;
		}

		/* sync on a frame */
		frame = sync_frame++ % vb.frames;
#if defined(BUZIOCSYNC)
		bs.frame = frame;
		res = ioctl(dev, BUZIOCSYNC, &bs);
		if (res < 0) {
			perror("BUZIOCSYNC");
			exit(1);
		}

		printf("frame %d, length=%u seq=%u stamp=%ld.%06ld\n",
		       sync_frame - 1, bs.length, bs.seq, bs.timestamp.tv_sec, bs.timestamp.tv_usec);

		/* write it out */
		if (write(file, &bs, sizeof(bs)) != sizeof(bs)
		    ||  write(file, buff + vb.offsets[frame], bs.length) != bs.length) {
			perror("write");
			exit(1);
		}
#else
		res = ioctl(dev, VIDIOCSYNC, &frame);
		if (res < 0) {
			perror("VIDIOCSYNC");
			exit(1);
		}

		/* write it out */
		if (write(file, &res, sizeof(res)) != sizeof(res)
		    ||  write(file, buff + vb.offsets[frame], res) != res) {
			perror("write");
			exit(1);
		}
#endif
	}

	res = close(file);
	if (res < 0) {
		perror(argv[1]);
		exit(1);
	}

	res = close(dev);
	if (res < 0) {
		perror("/dev/bttv0");
		exit(1);
	}

}
